""" Backend Super Module
    
This module defines the API that all the different operating system
modules MUST follow.
  
Each module has a single "helper" function "HostMatch()" that returns
true/false about whether this is the correct backend for this host.

There is also a single class "Backend" that implements the backend itself.

Gromit uses the HostMatch() calls in all the various modules to find the
correct backend, then instantiates ONE (1) Backend instance per run. (it
actually has a list of all available modules, and uses the first one whose
HostMatch() returns true)

The backend should be the simplest possible way to implement whatever
action. All the gromit specific crap (kickstart on or off, dependancies,
chroots, etc etc etc) SHOULD NOT BE HERE. Put that lot in rules.py or
something. We want new modules to be as easy to create as possible, and we
also want to be able to easily add gromit stuff in just one place.

In keeping with our simple is beatiful, backend methods are NOT allowed to
raise exceptions. (You can call things that throw exceptions, but you must
catch all possible exceptions.) You are also not allowed to return any error
values - either return a real value, (for get_os_release) or nothing at
all (service_config).

In case of an error condition, print an error on stderr and move on.

Note that this module is really just for documentation purposes: it might
be possible to include it, but that really isn't necessary and would probably
just complicate the namespaces.

"""

import socket


def HostMatch():
    """ Return a true/false value for whether this module matches the host. 
    
    A common idiom for implementing this is simply to test for the existence
    of some file that only appears on the system in question.

    HostMatch() takes no arguments and returns a simple true/false.

    A simple, typical sample implementation:

    import os.path
    if os.path.isfile("/etc/redhat-release"):
        return 1
    else:
        return 0

    """
    
    return 0

    
class Backend: 
    """ Backend Super Class
        
    This class defines the API that all the different operating system
    backend classes MUST follow.

    The current API list is:
    __init__():         constructor takes no args       
    __del__():          cleanup function takes no args (and may do nothing)

    General Information:
    get_hostname():     returns the Fully Qualified Domain Name (FQDN) if at
                        all possible.
    get_host_arch():    returns a simple arch string
    get_os_release():   returns a simple release string
    get_ip_number():    returns the dotted quad notation ip address. 

    Specific Information:
    is_pkg_installed(): takes a single string (pkg) and returns true/false
    
    Actually do stuff:
    service_config():   configures a daemon (initscript) to run/not run 
    service_control():  controls (starts/stops) a daemon without touching
                        its configuration
    """


    def __init__(self):
        """ Initialize the backend module
        
        Typically does things like go ahead and probe for anything that
        might be expensive to determine and asked for a lot (os version
        number, arch) and open any system databases needed (rpm or dpkg).
    
        """
    
        pass

    
    def __del__(self):
        """ Cleanup the backend. 
        
        If we opened a database, we could close it, etc.
        
        """
        
        pass
        
    
    def get_os_release(self):
        """ Return a simple string of what the OS release is. 
        
        In various rules we often want to control whether a rule is run or
        not dependant on which release of an OS we are.
        
        Typical values could be "7.1" "6.2" "7.0.91" for redhat systems or
        "2.7" (or "7") for solaris systems. The backend implementor should
        pick the most standard nomenclature. The name of the OS ("redhat" or
        "linux") should NOT be included. [I tend to try and return whatever
        RPM would have, but I'm redhat biased...]
        
        """
        
        return ""


    def get_host_arch(self):
        """ Return a simple string of what the hosts architecture is.
        
        This seemed a logical counterpart to get_os_release() - I'm not 
        sure of its utility.
                
        Typical values could be "i386" "sparc" "ppc" for redhat systems or
        "sun4u" (or "sun4") for solaris systems. The backend implementor
        should pick the most standard nomenclature. [I tend to try and
        return whatever RPM would have, but I'm redhat biased...]
        
        """
        
        return ""


    def get_ip_number(self):
        """ Return the hosts IP number as a simple dotted-quad string.
        
        Figuring out a hosts ip number (and its hostname) could be REALLY
        non-portable. I've included these two implementations as a
        default, assuming a) pythons networking code is ok, and b) you are
        using (and trusting) DNS. For me, these are both ALWAYS true if
        gromit is going to work anyway, but that may not be true at your
        site.
        
        """
        
        # this code is directly from the 1.5.2 doc, library ref section 7.2
        # FIXME: may need to delete or move /etc/hosts here, if we don't
        # want to trust it....
        ip = socket.gethostbyname(socket.gethostname())
        return ip
        
        
    def get_hostname(self):
        """ Return the hosts FQDN as a string.
        
        See the comments in get_ip_number() for more details.
        
        """
        
        # I got this code (and _getdnsnames and _resolve) out of the
        # python FAQTS site.  I don't claim to grok it.
        fullname = socket.gethostname()
        if '.' not in fullname:
            fullname = self._resolve(fullname)
        return fullname

             
    def is_pkg_installed(self, pkg):
        """ Return true/false for whether the pkg is installed on this host.
        
        In various rules we often want to control whether a rule is run or
        not dependant on whether a particular pkg is present (LPRng,
        sendmail).
        
        On most linux systems (esp RPM and dpkg based ones) this will be 
        very easy - query the database of pkgs. On other systems with weaker
        package management, I'm not sure what should be done. Solaris/HP-UX
        look like a real mess - you might have to specify a particular binary
        instead of a pkg, or do a filesystem query in /opt/pkgs or some such,
        which will be terrifyingly site specific.
  
        """
        
        return 0


    def service_control(self, service):
        """ Turn on/off a system service immediately.
        
        This typically means starting/stoping a daemon from the init
        scripts, but could be pretty platform dependant.
        
        """
        
        pass        

    
    def service_config(self, service):
        """ Configure a system service (daemon) for the next reboot or
        runlevel change.
        
        This typically means manipulating the init scripts in /etc for 
        the various runlevels. Obviously, highly system dependant.
        
        """
        
        pass        
    
    
    def _getdnsnames(self, name):
        """ A helper function, from the python FAQTS. """

        d = socket.gethostbyaddr(name)
        names = [ d[0] ] + d[1] + d[2]
        return names


    def _resolve(self, name):
        """ A helper function, from the python FAQTS. """

        names = self._getdnsnames(name)
        for dnsname in names:
            if '.' in dnsname:
                fullname = dnsname
                break
        else:
            fullname = name
        return fullname

    
## END OF LINE ###
