""" Gromit Config module

Gromit is a fairly configurable beast - conf file, command line, auto-probed
backends ... we need someplace to remember all this crap. The Config object
is it

"""

    
from exceptions import Exception
import getopt
import re
import string
import sys

import deps
import lib

class Config:

    """ The Config object contains all the non-rule related run time info.
    
    Config is expected to parse all the config file(s), command line, 
    figure out which backend to use, etc. Its basically ALL of gromit 
    except for the rule code.
    
    """
    
    # To make gromit work on another unix os, just add another module name
    # here that implements the same API as the redhat module.
    __BACKEND_MODULE_LIST = ("debian", "solaris", "redhat")
    

    def __init__(self, version_string, config_file, rules_file):

        # We parse the config file first so that it can hard-code a backend
        # These few initial defaults specify what entries are valid in the
        # gromit.conf file.
        self._gc = {}
        self._gc["host_keywords"] = ""
        self._gc["rules_file"] = rules_file
        self._gc["backend_modules"] = Config.__BACKEND_MODULE_LIST

        self._parse_config_file(config_file)

        # now we can put the version string in  - we don't want the conf
        # file overriding it above.
        self._gc["config_file"] = config_file
        self._gc["gromit_version"] = version_string
                
        # Auto-probe for what backend we should use.
        # Right now, there's no way to override a modules HostMatch() fn.
        # Should we ignore HostMatch() if the user wants us to?
        found = 0
        for name in Config.__BACKEND_MODULE_LIST:
            tmp = __import__(name)
            if tmp.HostMatch():
                found = 1
                break
                
        if not found:
            raise Exception("Could not find correct backend for this host.")
        else:
            self._backend = tmp.Backend()
            
        # Set the defaults in case they are not specified on the command line
        self._gc["kickstart"] = 0             ## default is not kickstart
        self._gc["chroot"] = "/"
        self._gc["dryrun"] = 0
        self._gc["files_dir"] = "./"
        self._gc["hosts_entries"] = ""        # used by append_hosts_text()
        
        self._parse_command_line()

        # Convert the type string into a dict we can use later
        self._gc["host_keywords"] = deps.ParseKeywordsDeps(self._gc["host_keywords"])

        # if the systems arch, release weren't overridden at the command line
        # or in the config file, use the backend to probe for them.
        if not self._gc.has_key("arch"):
            self._gc["arch"] = self._backend.get_host_arch()
        if not self._gc.has_key("release"):
            self._gc["release"] = self._backend.get_os_release()
            
    
    def _parse_command_line(self):
        """ parse command line
        
        command line overrides any other source of config options 
        
        """
    
        usage = """usage: gromit options
        -c | --config    config file to use 
             --chroot    use limited chroot (output) for testing
        -d | --debug     print out debugging statements
        -h | --help      print this help 
        -k | --kickstart act as if running in ks postinstall
        -n | --dryrun    don't _do_ anything, just print it
        -r | --rulefile  rules file to use 
        -t | --hostkeywords  specify which host keywords(s) to configure for
        -V | --version   print gromit version and exit"""
        
        usage_requested = 0
        
        try:
            opts, args = getopt.getopt(sys.argv[1:], "c:dhkt:nr:V", 
                         ("chroot=", "config=", "debug", "help", "kickstart", 
                          "keywords=", "dryrun", "rulefile=", "version"))

        except getopt.error, msg:
                sys.stdout = sys.stderr
                print msg
                print usage
                sys.exit(2)
   
        for o, a in opts:
                if o == "-c" or o == "--config":
                    self._gc["config_file"] = a
                if o == "--chroot":
                    self._gc["chroot"] = a
                if o == "-d" or o == "--debug":
                    # FIXME: Big time - this needs to do something USEFUL.
                    #DEBUG = 1
                    pass
                if o == "-r" or o == "--rulefile":
                    self._gc["rules_file"] = a
                if o == "-h" or o == "--help":
                    usage_requested = 1
                if o == "-k" or o == "--kickstart":
                    self._gc["kickstart"] = 1    
                if o == "-n" or o == "--dryrun":
                    self._gc["dryrun"] = 1
                if o == "-t" or o == "--hostkeywords": 
                    self._gc["host_keywords"] = a
                if o == "-V" or o == "--version":
                    print "gromit v%s (C) 2001 Hunter Matthews" % \
                        self._gc["gromit_version"]
                    print "Released under the GPL"
                    sys.exit(0)
        
        if usage_requested:
            print usage
            print "Config File =", self._gc["config_file"]
            print "Rules File =", self._gc["rules_file"]
            sys.exit(0)
        
    def _parse_config_file(self, config_file):
        """ parse the config file

        A constant location file that also contains some host specific bits.
        
        """

        linenum = 0

        # Regular expression to identify and pull apart a shell like var.    
        # FIXME: This re doesn't really describe shell syntax
        # varname, '=', value
        var_re = re.compile(
            r'(?P<varname>[-.\w]+)'               # - . _ alphanum
            r'='                                  # '=', followed
            r'"?(?P<varvalue>.*?)"?$'             # everything up to eol
            )
    
    
        # The conf file is optional - the defaults should be correct 
        # _most_ of the time.
        try:
            f = open(config_file, "r")
        except IOError, msg:
            # FIXME: should be a way to print a warning or error here,
            # but we haven't seen the command line yet...
            print "IOERROR trying to open config file"
            return
        
        
        while 1:
            line = f.readline()
            linenum = linenum + 1
            if not line:
                break
            
    #        lib.dbg_print("")
    #        lib.dbg_print("line = %s" % string.replace(line,"\n", ""))
    #        lib.dbg_print("linenum = %s" % linenum)        
    
            # whitespace is irrelevant
            line = string.strip(line)
            
            # Blank lines are ignored
            if len(line) == 0:
                #lib.dbg_print("Found an empty line")
                continue 
                
            # Comment line are '#' to the end of line, and ignored
            if line[0] == '#': 
                #lib.dbg_print("Found a comment")
                continue
            
            # actually parse out the variables.
            mo = var_re.match(line)
            if mo:
                varname, varvalue = mo.group('varname', 'varvalue')
                varname = string.lower(varname)
                if self._gc.has_key(varname):
                    self._gc[varname] = varvalue
                else:
                    raise Exception("Unrecognized variable '%s' at line %s" \
                                    % (varname, linenum))
    
            else:
                # parsing error.
                raise Exception("Invalid format for line number %s" % linenum)
                
        f.close()
        return    


    def __getitem__(self, item):
        if item == "backend":
            return self._backend
        else:
            return self._gc[item]
                

    def __setitem__(self, item, value):
        self._gc[item] = value
                

#    def __repr__(self):
#        print "__repr__() called"
#        return "config object"

        
#    def __str__(self):
#        print "__str__() called"
#        return "config object"

        
## END OF LINE ##
                
