""" Red Hat Linux Backend

Implements the Gromit Backend API for Red Hat Linux systems.

Please note: Because the author mostly admins Red Hat systems, this module
will be the most heavily used/tested (by far), and so represents the best
example API for the backends.

See backend.py for documentation for all the API calls.

"""

import os
import os.path
import rpm
import string
import sys

import backend


def HostMatch():
    if os.path.isfile("/etc/redhat-release"):
        return 1
    else:
        return 0
        

class Backend(backend.Backend):
    """ For proper documentation, see the base module backend.py. """

    def __init__(self):
        backend.Backend.__init__(self)
        
        # FIXME: This could throw all sorts of exceptions...
        # that would imply an exception model for the backends...
        self._rpm_db = rpm.opendb(0, "/")

        # get the release    
        index = self._rpm_db.findbyname("redhat-release")[0]
        self._release = self._rpm_db[index][rpm.RPMTAG_VERSION]

        # get the arch
        f = os.popen("/bin/uname -m", "r")
        result = string.strip(f.readline())
        f.close()
        
        # uname reports the exact arch - i686, i586, usparc. We really want
        # the arch "family" that the host belongs to.
        self._arch = self._get_arch_family(result)
        
        
    def __del__(self):
        # rpmdb doesn't have a "close" method that I can find - maybe 
        # an explicit __del__ is implemented.
        del self._rpm_db
        
    
    def get_os_release(self):
        return self._release        


    def get_host_arch(self):
        return self._arch
        
        
    def is_pkg_installed(self, pkg):
        # FIXME: I used to have a caching system to avoid multiple 
        # try/except series for a single pkg, but threw it out to simpify
        # the implementation.

        #dbg_print("redhat.is_pkg_installed(%s)" % pkg)
            
        try: 
            found = self._rpm_db.findbyname(pkg)
        except (rpm.error, IndexError):
            #dbg_print("    failed db lookup")
            return 0
        else:
            ## WART 
            ## Red Hat changed the API at 4.0.2  - findbyname() will not
            ##   throw an exception but instead returns an empty list.
            ## More logical, but highly annoying in a micro version rev.
            if len(found) == 0:
                #dbg_print("   failed db lookup")
                return 0
            else:
                #dbg_print("    succedded db lookup")
                return 1



    def service_control(self, service, action):
        # start stop restart condrestart status reload panic save 
        # start and stop to begin with, add reload/restart later

        if action != "start" and action != "stop":
            sys.stderr.write("ERROR: service_control() action must be " +
                             "either 'start' or 'stop'\n")
            return

        # Don't start already running services, don't stop ones not running
        if os.path.exists("/var/lock/subsys/%s" % service):
            if action == "start":
                return
        else:
            if action == "stop":
                return

        if os.system("/sbin/service %s %s > /dev/null 2>&1" % 
                     (service, action)):
            sys.stderr.write("ERROR: \"/sbin/service %s %s\" returned an " \
                             "error\n" % (service, action))
            return


    def service_config(self, service, action):
        # --add --del, --level, on/off/reset
        # on/off to begin with, --level later

        if action != "on" and action != "off":
            sys.stderr.write("ERROR: service_config() action must be " +
                             "either 'on' or 'off'\n")
            return

        if os.system("/sbin/chkconfig %s %s > /dev/null 2>&1" % 
                     (service, action)):
            sys.stderr.write("ERROR: /sbin/chkconfig on %s returned an " +
                             "error of %s\n")
            return
        

    def _get_arch_family(self, arch):
        # I wish there were a more efficient, less maintenence prone way to
        # do this, but I can't think of one.
        
        # The code for this was tortured from wallace's
        # pkgDB._getCompatRepositoryList()
        
        # family on the left, members of that family on the right. 
        # dict needs to contain any string "uname -m" can return.
        repcompat = {}
        repcompat['i386'] = ['athlon', 'i686', 'i586', 'i486', 'i386' ]
        repcompat['sparc'] = [ 'usparc', 'sparc64', 'sparc' ]
        repcompat['alpha'] = [ 'alphaev67', 'alphaev6', 'alphapca56', 'alphaev56', 
                               'alphaev5', 'alpha', 'axp' ]
        repcompat['ppc'] = [ 'ppc' ]
        
        for family in repcompat.keys():
            if arch in repcompat[family]:
                return family
                
        return ""


## END OF LINE ##
