""" Gromit Rules Library

This modules defines the calls that the users rules file may use. (An API).
Since the API must be portable to all hosts (unix hosts), anything not
provided by the python basic libraries must be implemented in the backend
module. (We trust that python is portable - if thats not true, I quit)

So for instance, create_dir() can simply use the python provided
os.makedirs(), but set_service() must use functions within the backends.

So far the following API is available:
    create_dir()          creates a directory       (I started small... :)
    set_files_dir()       specifies the location to pull copied files from
    install_file()        copy a (config) file into place 
                          this is the real workhorse of gromit
    set_service()         turn a system service (daemon usually) on or off

See the sample gromit.rules file for detailed documentation.    

"""

import os
import os.path
import shutil
import stat
import string
import sys
    
import lib
import deps

# gromit_config needs to be a "global" object in this module's namespace
gromit_config = None


def create_dir(dir_name, 
               pkg_dep="", release="", keywords="", 
               mode="0755"):
    """ Create a directory. I started simple. :) """
    
    # deal with chroot, if any
    cr_dir_name = lib.PathJoin(gromit_config["chroot"], dir_name)

    # decide whether to do anything at all    
    if not deps.CheckDependancies(pkg_dep, release, keywords):
        return

    # if the dir is there, do nothing, report nothing
    if os.path.isdir(cr_dir_name):
        if gromit_config["dryrun"]:
            print "directory %s already exists" % cr_dir_name
        return

    # dir isn't there, but something with the same name is
    elif os.path.exists(cr_dir_name):
        sys.stderr.write("WARNING: %s exists, but is not a directory\n" %
                      cr_dir_name)
        return
    
    # real case, make the dir
    else:
        if gromit_config["dryrun"]:
            print "creating %s, mode = %s" % (cr_dir_name, mode)
        else:
            os.makedirs(cr_dir_name, string.atoi(mode, 8))


def set_files_dir(files_dir):
    """ Set the directory to "install_file()" from. """
    
    gromit_config["files_dir"] = files_dir

    
def install_file(filename, path, 
                 pkg_dep="", release="", keywords="",
                 mode=""):
    """ install a file (usually a config file) """

    # decide whether to do anything at all    
    if not deps.CheckDependancies(pkg_dep, release, keywords):
        return

    # actually do something
    from_path = lib.PathJoin(gromit_config["files_dir"], filename)
    to_path = lib.PathJoin(gromit_config["chroot"], path)

    # "" as our mode means to use the mode of the file in files_dir
    if mode == "":
        try:
            mode = stat.S_IMODE(os.stat(from_path)[0])
        except (IOError, OSError), ex:
            sys.stderr.write("ERROR: could not stat file: %s\n" % ex)
            return
    else:
        mode = int(mode)
        
    if gromit_config["dryrun"]:
        if lib.Md5File(from_path) != lib.Md5File(to_path):
            print "installing %s to %s, mode %s"  % (from_path, to_path, mode)
        else:
            print "not installing %s to %s short-circuited: md5's match" % \
                (from_path, to_path)
    else:
        if lib.Md5File(from_path) != lib.Md5File(to_path):
            try:
                create_dir(os.path.dirname(path))
                shutil.copy(from_path, to_path)
                os.chmod(to_path, mode)
            except (OSError, IOError), ex:
                sys.stderr.write("ERROR: could not copy file: %s\n" % ex)
    

def set_service(service, state, pkg_dep="", release="", keywords=""):
    """ Try to turn off or on certain services (daemons)

    For this function, chroot means the same as dryrun.
    
    """

    # For this function, we assume the server name is also the pkg name if
    # the pkg_dep is the keyword ""
    if pkg_dep == "":
        pkg_dep = service
    
    if not deps.CheckDependancies(pkg_dep, release, keywords):
        return
    
    # service_control needs a different action string.
    if state == "on":
        control_action = "start"
    elif state == "off":
        control_action = "stop"
    else:
        sys.stderr.write("ERROR: set_service() should only be called with" +
                         "state set to 'on' or 'off'.")

    # configure the service (probably init scripts)     
    if gromit_config["chroot"] != "/" or gromit_config["dryrun"]:
        print "backend.service_config(%s, %s)" % (service, state)
    else:
        gromit_config["backend"].service_config(service, state)

    # control the service (probably run or kill a daemon)
    if not gromit_config["kickstart"]:
        if gromit_config["chroot"] != "/" or gromit_config["dryrun"]:
            # we really need the "status" keyword here...
            # but I only know for sure that redhat provides that
            print "backend.service_control(%s, %s)" % (service, control_action)
        else:
            gromit_config["backend"].service_control(service, control_action)

    
#def setup_hosts_file():
    # Used to be a function to "normalize" the /etc/hosts file, but 
    # proved virtually unworkable in a sane manner. It also proved almost
    # useless - with few exceptions, /etc/hosts is only done once.

    # Functionality now done in the post install of kickstart

## END OF LINE ##
